"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _utils = require("./utils");

var _pathToRegexp = _interopRequireDefault(require("path-to-regexp"));

var _queryString = _interopRequireDefault(require("query-string"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var defaultPathPattern = /.*/;

var defaultMatch = function defaultMatch() {
  return false;
};

var paramDefaults = {
  parse: function parse(value) {
    return value;
  },
  stringify: function stringify(value) {
    return String(value);
  },
  notSetValue: null
};

var isParamOptional = function isParamOptional(param, pathParams) {
  var pathParam = pathParams.find(function (pathParam) {
    return pathParam.name === param.name;
  });
  return pathParam ? pathParam.optional : true;
};

var createParamConditions = function createParamConditions(params) {
  var paramNames = params.map(function (param) {
    return param.name;
  });
  return {
    isPathParam: function isPathParam(name) {
      return paramNames.includes(name);
    },
    isQueryParam: function isQueryParam(name) {
      return !paramNames.includes(name);
    }
  };
};

var createPathPattern = function createPathPattern(path, pathParams) {
  return path !== undefined ? (0, _pathToRegexp.default)(path, pathParams) : defaultPathPattern;
};

var createParams = function createParams() {
  var params = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  var pathParams = arguments.length > 1 ? arguments[1] : undefined;
  return Object.entries(params).map(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        key = _ref2[0],
        param = _ref2[1];

    return _objectSpread({
      key: key
    }, paramDefaults, param);
  }).map(function (param) {
    return [param.key, {
      key: param.key,
      name: param.name,
      action: param.action,
      selector: param.selector,
      parse: param.parse,
      stringify: param.stringify,
      notSetValue: param.notSetValue,
      optional: isParamOptional(param, pathParams)
    }];
  }).reduce(_utils.createObject, {});
};

var createMatch = function createMatch() {
  var match = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : defaultMatch;
  var params = arguments.length > 1 ? arguments[1] : undefined;
  return function (state) {
    var baseMatch = match(state);
    var requiredMatch = Object.values(params).filter(function (param) {
      return !param.optional;
    }).every(function (param) {
      return param.selector(state) !== param.notSetValue;
    });
    return baseMatch && requiredMatch;
  };
};

var createGetLocation = function createGetLocation(path, pathParams) {
  if (path !== undefined) {
    var _createParamCondition = createParamConditions(pathParams),
        isPathParam = _createParamCondition.isPathParam,
        isQueryParam = _createParamCondition.isQueryParam;

    var getPath = _pathToRegexp.default.compile(path);

    var getSearch = _queryString.default.stringify;
    return function (values, location) {
      return (0, _utils.normalizeLocation)(_objectSpread({}, location, {
        pathname: getPath((0, _utils.keyFilter)(values, isPathParam)),
        search: getSearch((0, _utils.keyFilter)(values, isQueryParam))
      }));
    };
  } else {
    return function (values, location) {
      return location;
    };
  }
};

var createGetParamValues = function createGetParamValues(pathPattern, pathParams) {
  var _createParamCondition2 = createParamConditions(pathParams),
      isPathParam = _createParamCondition2.isPathParam,
      isQueryParam = _createParamCondition2.isQueryParam;

  var getPathValues = function getPathValues(location) {
    var result = pathPattern.exec(location.pathname);
    var values = result.splice(1);
    return (0, _utils.zip)(pathParams, values).filter(function (_ref3) {
      var _ref4 = _slicedToArray(_ref3, 2),
          value = _ref4[1];

      return value !== undefined;
    }).map(function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          param = _ref6[0],
          value = _ref6[1];

      return [param.name, decodeURIComponent(value)];
    }).reduce(_utils.createObject, {});
  };

  var getSearchValues = function getSearchValues(location) {
    return _queryString.default.parse(location.search);
  };

  return function (location) {
    return _objectSpread({}, (0, _utils.keyFilter)(getPathValues(location), isPathParam), (0, _utils.keyFilter)(getSearchValues(location), isQueryParam));
  };
};

var createLocationMatch = function createLocationMatch(pathPattern) {
  return function (location) {
    return pathPattern.test(location.pathname);
  };
};

var _default = function _default(routes) {
  return routes.map(function (route) {
    var pathParams = [];
    var pathPattern = createPathPattern(route.path, pathParams);
    var params = createParams(route.params, pathParams);
    return {
      key: route.key,
      path: route.path,
      redirect: route.redirect,
      action: route.action,
      match: createMatch(route.match, params),
      getLocation: createGetLocation(route.path, pathParams),
      getParamValues: createGetParamValues(pathPattern, pathParams),
      locationMatch: createLocationMatch(pathPattern),
      params: params
    };
  });
};

exports.default = _default;