"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createSpec = exports.getRoute = exports.redirect = exports.keyFilter = exports.createObject = exports.zip = exports.locationEqualsPath = exports.locationsEqual = exports.normalizeLocation = void 0;

var _history = require("history");

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var normalizeLocation = function normalizeLocation(location) {
  return (0, _history.parsePath)((0, _history.createPath)(location));
};

exports.normalizeLocation = normalizeLocation;

var locationsEqual = function locationsEqual(locationA, locationB) {
  return locationA.pathname === locationB.pathname && locationA.search === locationB.search && locationA.hash === locationB.hash;
};

exports.locationsEqual = locationsEqual;

var locationEqualsPath = function locationEqualsPath(location, path) {
  return locationsEqual(location, (0, _history.parsePath)(path));
};

exports.locationEqualsPath = locationEqualsPath;

var zip = function zip(a, b) {
  var zipped = [];

  for (var index = 0; index < a.length && index < b.length; index++) {
    zipped.push([a[index], b[index]]);
  }

  return zipped;
};

exports.zip = zip;

var createObject = function createObject(object, _ref) {
  var _ref2 = _slicedToArray(_ref, 2),
      key = _ref2[0],
      value = _ref2[1];

  object[key] = value;
  return object;
};

exports.createObject = createObject;

var keyFilter = function keyFilter(object, condition) {
  return Object.keys(object).filter(condition).map(function (key) {
    return [key, object[key]];
  }).reduce(createObject, {});
};

exports.keyFilter = keyFilter;

var redirect = function redirect(routes, route) {
  if (route !== undefined && route.redirect !== undefined) {
    var redirected = routes.find(function (other) {
      return other.key === route.redirect;
    });
    return redirect(routes, redirected);
  } else {
    return route;
  }
};

exports.redirect = redirect;

var getRoute = function getRoute(routes, key) {
  var route = routes.find(function (route) {
    return route.key === key;
  });
  return redirect(routes, route);
};

exports.getRoute = getRoute;

var createSpec = function createSpec(route) {
  var values = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var paramValues = Object.values(route.params).map(function (_ref3) {
    var key = _ref3.key,
        notSetValue = _ref3.notSetValue;
    var value = key in values ? values[key] : notSetValue;
    return [key, value];
  }).reduce(createObject, {});
  return {
    route: route,
    params: paramValues
  };
};

exports.createSpec = createSpec;